from typing import Literal
from pydantic_xml import BaseXmlModel, attr


class Athlete(BaseXmlModel, tag="ATHLETE"):
    firstname: str = attr(name="firstname")
    lastname: str = attr(name="lastname")
    gender: Literal["M", "F", "X"] = attr(name="gender")
    birthdate: str = attr(name="birthdate")
    club: str = attr(name="club")
    time: str = attr(name="time")
    heatnum: int = attr(name="heatnum")
    lanenum: int = attr(name="lanenum")
    entrytime: str = attr(name="entrytime")
    starttime: str = attr(name="starttime")
    completeddistance: int = attr(name="completeddistance")
    timemodified: str = attr(name="timemodified")
    disqualification: str = attr(name="disqualification")



from typing import List, Literal
from pydantic_xml import BaseXmlModel, attr, element

from .athlete import Athlete


class Event(BaseXmlModel, tag="EVENT"):
    name: str = attr(name="name")
    gender: Literal["M", "F", "X"] = attr(name="gender")
    stroke: str = attr(name="stroke")
    distance: int = attr(name="distance")
    heatcount: int = attr(name="heatcount", default=0)
    relaycount: int = attr(name="relaycount", default=0)

    athletes: List[Athlete] = element(tag="ATHLETE", default_factory=list)



from __future__ import annotations
from lxml import etree

from pathlib import Path
from typing import Union
from .meet import Meet


PathLike = Union[str, Path]


def fromfile(path: PathLike) -> Meet:
    """
    Read XML from file and parse it into Meet.
    Important: pass BYTES to from_xml() to support XML declaration with encoding.
    """
    p = Path(path)
    xml_bytes: bytes = p.read_bytes()
    return Meet.from_xml(xml_bytes)


def tofile(
    meet: Meet,
    path: PathLike,
    *,
    encoding: str = "utf-8",
    pretty: bool = True,
    indent: int = 4,
    skip_empty: bool = True,
    exclude_none: bool = True,
    exclude_unset: bool = False,
) -> None:
    """
    Serialize Meet to XML and write it to file.
    NOTE: meet.to_xml() delegates to lxml.etree.tostring(), so do NOT pass pretty_print/indent there.
    """
    p = Path(path)

    # 1) дерево
    root = meet.to_xml_tree(
        skip_empty=skip_empty,
        exclude_none=exclude_none,
        exclude_unset=exclude_unset,
    )

    if pretty:
        try:
            etree.indent(root, space=" " * indent)
        except TypeError:
            etree.indent(root)

    xml_bytes: bytes = etree.tostring(
        root,
        encoding=encoding,
        xml_declaration=True,
        method="xml",
    )

    p.write_bytes(xml_bytes)



from typing import List, Literal
from pydantic_xml import BaseXmlModel, attr, element

from .event import Event


class Meet(BaseXmlModel, tag="MEET"):
    name: str = attr(name="name")
    year: int = attr(name="year")
    course: Literal["SCM", "LSM"] = attr(name="course")
    lanecount: int = attr(name="lanecount")
    timingdistance: int = attr(name="timingdistance")
    feventsagegroups: str = attr(name="feventsagegroups")
    meventsagegroups: str = attr(name="meventsagegroups")
    xeventsagegroups: str = attr(name="xeventsagegroups")
    timestandardfilename: str = attr(name="timestandardfilename")
    disqualificationcodes: str = attr(name="disqualificationcodes")

    events: List[Event] = element(tag="EVENT", default_factory=list)



from setuptools import setup, find_packages


setup(
    name='dianpy',
    description='DianPy a special parser for working with the dian scoreboard',
    version='1.0.2',
    install_requires=[
        'pydantic-xml>=2.18.0',
        'lxml>=6.0.2'
    ],
    packages=find_packages()
)
